import java.io.Serializable;
import java.sql.*;
import java.util.*;

public class Model implements Serializable{
	
	//Campos de configuracion
	
	private String JDBCDriver;
	private String JDBCUrl;
	private transient Connection con;
	
	//Campos de Operaciones
	
	private List operaciones;
	private String codigoOperacion;
	
	
	//Campos de Estadistica
	
	private List estadisticas;
	private String codigoEstadistica;
	
	
	//Campos de Sede
	
	private List sedes;
	private String codigoSede;
	
	
	//Campos de Caja
	
	private List cajas;
	private String codigoCaja;
	
	
	//Campos de Usuario
	
	private List usuarios;
	private String codigoUsuario;
	
	
	//Campos de Empleado
	
	private List empleados;
	private String codigoEmpleado;
	
	
	//Campos de Concepto
	
	private List conceptos;
	private String codigoConcepto;
	
	
	
	
	//connect(): conecta a la base de datos
	
	public void connect()throws SQLException{
		if(isConnected())
			throw new SQLException("Ya se encuentra conectado");
		if(JDBCDriver==null)	
			throw new SQLException("No hay driver cargado");
		if(JDBCUrl == null)
			throw new SQLException("No hay URL Especificada");
		
		try{
			Class.forName(JDBCDriver);
		}catch(Exception e){
			throw new SQLException("Error al cargar el driver "+JDBCDriver);
		}finally{
			con = DriverManager.getConnection(JDBCUrl);
		}
		
	}
	
	//isConntected(): Verifica si hay o no conexion ya establecida
	
	public boolean isConnected(){
		return (con!=null);
	}
	
	//disconnect(): Desconecta de la base de datos
	
	public void disconnect(){
		if(con != null){
			try{
				con.close();
			}catch(SQLException e){
				
			}finally{
				con = null;
			}
		}
	}
	
	
	/*
	 *
	 *getter and setter
	 *
     */
     
     //Asigna al Driver de JDBC
     
	public void setJDBCDriver(String JDBCDriver){
		this.JDBCDriver = JDBCDriver;
	}
	
	//Asigna la Url
	
	public void setJDBCUrl(String JDBCUrl){
		this.JDBCUrl = JDBCUrl;
	}
	
	//Especifica el codigo
	
	public void setCodigoOperacion(String codigoOperacion){
		this.codigoOperacion = codigoOperacion;
	}
	
	//Retorna el Driver
	
	public String getJDBCDriver(){
		return this.JDBCDriver;
	}
	
	//Retorna la Url
	
	public String getJDBCUrl(){
		return this.JDBCUrl;
	}
	
	//Retorna el codigo
	
	public String getCodigoOperacion(){
		return this.codigoOperacion;
	}
	
	/******************************************************************
	 ***********************Actualizar Operacion**************************
	 ******************************************************************/
	
	public void updateOperacion(Operacion operacion)throws Exception{
		if(!isConnected())
			throw new SQLException("No hay conexion");
		PreparedStatement pstmt = null;
		
		try{
			pstmt = con.prepareStatement("UPDATE Operaciones SET cantidad=?, tipo=?, fecha=?, codigoEmpleado=?, codigoCaja=?, codigoConcepto=? WHERE codigo=?");
			pstmt.setString(1,operacion.getCantidad());
			pstmt.setString(2,operacion.getTipo());
			pstmt.setDate(3,operacion.getFecha());
			pstmt.setString(4,operacion.getCodigoEmpleado());
			pstmt.setString(5,operacion.getCodigoCaja());
			pstmt.setString(6,operacion.getCodigoConcepto());
			pstmt.setString(7,operacion.getCodigo());
			
			pstmt.execute();
		}catch(SQLException e){
			System.out.println(e.getMessage());
		}finally{
			if(pstmt != null)
				pstmt.close();
		}
	}
	
	/******************************************************************
	 ***********************Eliminar Operacion**************************
	 ******************************************************************/
	
	public void delOperacion(Operacion operacion)throws Exception{
		if(!isConnected())
			throw new SQLException("No hay conexion");
		PreparedStatement pstmt = null;
		
		try{
			pstmt = con.prepareStatement("DELETE FROM Operaciones WHERE codigo = ?");
			pstmt.setString(1, operacion.getCodigo());
			
			pstmt.execute();
		}catch(SQLException e){
			System.out.println(e.getMessage());
		}finally{
			if(pstmt != null)
				pstmt.close();
		}
	}
	
	/******************************************************************
	 ***********************Agregar Operacion**************************
	 ******************************************************************/
	
	public void addOperacion(Operacion operacion)throws Exception{
		if(!isConnected())
			throw new SQLException("No hay conexion");
		PreparedStatement pstmt = null;
		
		try{
			pstmt = con.prepareStatement("INSERT INTO Operaciones VALUES(?,?,?,?,?,?,?)");
			pstmt.setString(1,operacion.getCodigo());
			pstmt.setString(2,operacion.getCantidad());
			pstmt.setString(3,operacion.getTipo());
			pstmt.setDate(4,operacion.getFecha());
			pstmt.setString(5,operacion.getCodigoEmpleado());
			pstmt.setString(6,operacion.getCodigoCaja());
			pstmt.setString(7,operacion.getCodigoConcepto());
			
			pstmt.execute();
		}catch(SQLException e){
			System.out.println(e.getMessage());
		}finally{
			if(pstmt != null)
				pstmt.close();
		}
	}
	//Retorna un objeto Operacion de acuerdo al codigo dado
	public Estadistica getEstadistica()throws Exception{
		if(!isConnected())
			throw new SQLException("No hay conexion");
		if(codigoEstadistica == null)
			throw new SQLException("No hay codigo de estadistica");
		
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		Estadistica estadistica = null;
		
		try{
			pstmt = con.prepareStatement("SELECT*FROM Estadisticas WHERE codigo=?");
			pstmt.setString(1,codigoEstadistica);
			
			rs = pstmt.executeQuery();
			
			if(rs.next())
				estadistica = Estadistica.load(rs);
		}finally{
			if(rs!= null)
				rs.close();
			if(pstmt != null)
				pstmt.close();
		}
		return estadistica;
	}
	
	public Sede getSede()throws Exception{
		if(!isConnected())
			throw new SQLException("No hay conexion");
		if(codigoSede == null)
			throw new SQLException("No hay codigo de Sede");
		
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		Sede sede = null;
		
		try{
			pstmt = con.prepareStatement("SELECT*FROM Sedes WHERE codigo=?");
			pstmt.setString(1,codigoSede);
			
			rs = pstmt.executeQuery();
			
			if(rs.next())
				sede = Sede.load(rs);
		}finally{
			if(rs != null)
				rs.close();
			if(pstmt != null)
				rs.close();
		}
		
		return sede;
	}
	
	public Caja getCaja()throws Exception{
		if(!isConnected())
			throw new SQLException("No hay conexion");
		if(codigoCaja == null)
			throw new SQLException("No hay codigo de Caja");
		
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		Caja caja = null;
		
		try{
			pstmt = con.prepareStatement("SELECT*FROM Cajas WHERE codigo=?");
			pstmt.setString(1,codigoCaja);
			
			rs = pstmt.executeQuery();
			if(rs.next())
				caja = Caja.load(rs);
		}finally{
			if(rs != null)			
				rs.close();
			if(pstmt != null)
				pstmt.close();
		}
		return caja;
	}
	
	public Usuario getUsuario()throws Exception{
		if(!isConnected())
			throw new SQLException("No hay conexion");
		if(codigoOperacion == null)
			throw new SQLException("No hay codigo de Usuario");
		
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		Usuario usuario = null;
		
		try{
			pstmt = con.prepareStatement("SELECT*FROM Usuarios WHERE codigo=?");
			pstmt.setString(1,codigoUsuario);
			
			//Ejecutamos consulta
			
			rs = pstmt.executeQuery();
			if(rs.next())
				usuario = Usuario.load(rs);
		}finally{
			if(rs != null)
				rs.close();
			if(pstmt != null)
				pstmt.close();
		}
		return usuario;
	}
	
	public Empleado getEmpleado()throws Exception{
		if(!isConnected())
			throw new SQLException("No hay conexion");
		if(codigoOperacion == null)
			throw new SQLException("No hay codigo de Empleado");
		
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		Empleado empleado = null;
		
		try{
			pstmt = con.prepareStatement("SELECT*FROM Empleados WHERE codigo=?");
			pstmt.setString(1,codigoEmpleado);
			
			//Ejecutamos consulta
			
			rs = pstmt.executeQuery();
			if(rs.next())
				empleado = Empleado.load(rs);
		}finally{
			if(rs != null)
				rs.close();
			if(pstmt != null)
				pstmt.close();
		}
		return empleado;
	}
	
	public Concepto getConcepto()throws Exception{
		if(!isConnected())
			throw new SQLException("No hay conexion");
		if(codigoOperacion == null)
			throw new SQLException("No hay codigo de Concepto");
		
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		Concepto concepto = null;
		
		try{
			pstmt = con.prepareStatement("SELECT*FROM Conceptos WHERE codigo=?");
			pstmt.setString(1,codigoConcepto);
			
			//Ejecutamos consulta
			
			rs = pstmt.executeQuery();
			if(rs.next())
				concepto = Concepto.load(rs);
		}finally{
			if(rs != null)
				rs.close();
			if(pstmt != null)
				pstmt.close();
		}
		return concepto;
	}
	
	public Operacion getOperacion()throws Exception{
		if(!isConnected())
			throw new SQLException("No hay conexion");
		if(codigoOperacion == null)
			throw new SQLException("No hay codigo de operacion");
		
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		Operacion operacion = null;
		
		try{
			pstmt = con.prepareStatement("SELECT*FROM Operaciones WHERE codigo=?");
			pstmt.setString(1,codigoOperacion);
			
			//Ejecutamos consulta
			
			rs = pstmt.executeQuery();
			if(rs.next())
				operacion = Operacion.load(rs);
		}finally{
			if(rs != null)
				rs.close();
			if(pstmt != null)
				pstmt.close();
		}
		return operacion;
	}
	
	//Devuelve una List con todas las Operaciones buscadas
	public List getOperaciones(){
		return operaciones;
	}
	public List getCajas(){
		return cajas;
	}
	public List getSedes(){
		return sedes;
	}
	public List getUsuarios(){
		return usuarios;
	}
	public List getEmpleados(){
		return empleados;
	}
	public List getEstadisticas(){
		return estadisticas;
	}
	public List getConceptos(){
		return conceptos;
	}
	
	
	//Actualizar
	public void searchOperacion(String solicitante)throws SQLException{
		if(!isConnected())
			throw new SQLException("No hay conexion");
		
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		operaciones = null;
		
		try{
			
			pstmt = con.prepareStatement("SELECT * FROM Operaciones WHERE solicitante like ? ORDER BY codigo");
			solicitante = solicitante.trim();
			solicitante = "%"+solicitante+"%";
			
			pstmt.setString(1,solicitante);
			
			rs = pstmt.executeQuery();
			operaciones = new LinkedList();
			while(rs.next()){
				operaciones.add(Operacion.load(rs));
			}
		}finally{
			if(rs!=null)
				rs.close();
			if(pstmt != null)
				pstmt.close();
		}
	}
	
}

